@@ -1,6 +1,6 @@
 /* BFD Support for the ADI Blackfin processor.
 
-   Copyright (C) 2005-2017 Free Software Foundation, Inc.
+   Copyright (C) 2005-2022 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -25,17 +25,18 @@
 
 const bfd_arch_info_type bfd_bfin_arch =
   {
-    16,     		/* Bits in a word.  */
-    32,  		/* Bits in an address.  */
-    8,     		/* Bits in a byte.  */
+    16,			/* Bits in a word.  */
+    32,			/* Bits in an address.  */
+    8,			/* Bits in a byte.  */
     bfd_arch_bfin,
-    0,                	/* Only one machine.  */
-    "bfin",        	/* Arch name.  */
-    "bfin",        	/* Arch printable name.  */
-    4,                	/* Section align power.  */
-    TRUE,             	/* The one and only.  */
+    0,			/* Only one machine.  */
+    "bfin",		/* Arch name.  */
+    "bfin",		/* Arch printable name.  */
+    4,			/* Section align power.  */
+    true,		/* The one and only.  */
     bfd_default_compatible,
     bfd_default_scan,
     bfd_arch_default_fill,
-    0,
+    NULL,
+    0 /* Maximum offset of a reloc from the start of an insn.  */
   };
