@@ -1,5 +1,5 @@
 /* coff object file format
-   Copyright (C) 1989-2020 Free Software Foundation, Inc.
+   Copyright (C) 1989-2022 Free Software Foundation, Inc.
 
    This file is part of GAS.
 
@@ -41,12 +41,8 @@
 #endif
 
 #ifdef TC_PPC
-#ifdef TE_PE
-#include "coff/powerpc.h"
-#else
 #include "coff/rs6000.h"
 #endif
-#endif
 
 #ifdef TC_I386
 #ifdef TE_PEP
@@ -133,10 +129,8 @@
 
 #define OUTPUT_FLAVOR bfd_target_coff_flavour
 
-/* Alter the field names, for now, until we've fixed up the other
-   references to use the new name.  */
+/* COFF symbol flags.  See SF_* macros.  */
 #define OBJ_SYMFIELD_TYPE	unsigned long
-#define sy_obj			sy_obj_flags
 
 /* We can't use the predefined section symbols in bfd/section.c, as
    COFF symbols have extra fields.  See bfd/libcoff.h:coff_symbol_type.  */
@@ -184,9 +178,15 @@
 #define SA_SET_SCN_NRELOC(s,v)	(SYM_AUXENT (s)->x_scn.x_nreloc = (v))
 #define SA_SET_SCN_NLINNO(s,v)	(SYM_AUXENT (s)->x_scn.x_nlinno = (v))
 
-/* Internal use only definitions. SF_ stands for symbol flags.
+#ifdef OBJ_XCOFF
+#define SA_GET_SECT_SCNLEN(s)	(SYM_AUXENT (s)->x_sect.x_scnlen)
+#define SA_GET_SECT_NRELOC(s)	(SYM_AUXENT (s)->x_sect.x_nreloc)
+#define SA_SET_SECT_SCNLEN(s,v)	(SYM_AUXENT (s)->x_sect.x_scnlen = (v))
+#define SA_SET_SECT_NRELOC(s,v)	(SYM_AUXENT (s)->x_sect.x_nreloc = (v))
+#endif
 
-   These values can be assigned to sy_symbol.ost_flags field of a symbolS.  */
+/* Internal use only definitions.  SF_ stands for symbol flags.  These
+   values can be assigned to OBJ_SYMFIELD_TYPE obj field of a symbolS.  */
 
 #define SF_NORMAL_MASK	0x0000ffff	/* bits 12-15 are general purpose.  */
 
@@ -246,7 +246,7 @@ extern int coff_n_line_nos;
 extern symbolS *coff_last_function;
 
 #define obj_emit_lineno(WHERE, LINE, FILE_START)	abort ()
-#define obj_app_file(name, app)      c_dot_file_symbol (name, app)
+#define obj_app_file(name)           c_dot_file_symbol (name)
 #define obj_frob_symbol(S,P) 	     coff_frob_symbol (S, & P)
 #define obj_frob_section(S)	     coff_frob_section (S)
 #define obj_frob_file_after_relocs() coff_frob_file_after_relocs ()
@@ -284,7 +284,7 @@ extern const pseudo_typeS coff_pseudo_table[];
    as in start/_start/__start in gcc/libgcc1-test.c.  */
 #define RESOLVE_SYMBOL_REDEFINITION(sym)		\
 (SF_GET_GET_SEGMENT (sym)				\
- ? (sym->sy_frag = frag_now,				\
+ ? (sym->frag = frag_now,				\
     S_SET_VALUE (sym, frag_now_fix ()),			\
     S_SET_SEGMENT (sym, now_seg),			\
     0)							\
@@ -298,17 +298,26 @@ extern const pseudo_typeS coff_pseudo_table[];
 #define INIT_STAB_SECTION(seg) obj_coff_init_stab_section (seg)
 
 /* Store the number of relocations in the section aux entry.  */
+#ifdef OBJ_XCOFF
+#define SET_SECTION_RELOCS(sec, relocs, n)		\
+  do {							\
+    symbolS * sectSym = section_symbol (sec);		\
+    if (S_GET_STORAGE_CLASS (sectSym) == C_DWARF)	\
+      SA_SET_SECT_NRELOC (sectSym, n);			\
+    else						\
+      SA_SET_SCN_NRELOC (sectSym, n);			\
+  } while (0)
+#else
 #define SET_SECTION_RELOCS(sec, relocs, n) \
   SA_SET_SCN_NRELOC (section_symbol (sec), n)
-
-#define obj_app_file(name, app) c_dot_file_symbol (name, app)
+#endif
 
 extern int  S_SET_DATA_TYPE              (symbolS *, int);
 extern int  S_SET_STORAGE_CLASS          (symbolS *, int);
 extern int  S_GET_STORAGE_CLASS          (symbolS *);
 extern void SA_SET_SYM_ENDNDX            (symbolS *, symbolS *);
 extern void coff_add_linesym             (symbolS *);
-extern void c_dot_file_symbol            (const char *, int);
+extern void c_dot_file_symbol            (const char *);
 extern void coff_frob_symbol             (symbolS *, int *);
 extern void coff_adjust_symtab           (void);
 extern void coff_frob_section            (segT);
@@ -324,7 +333,6 @@ extern void pecoff_obj_clear_weak_hook   (symbolS *);
 extern void obj_coff_section             (int);
 extern segT obj_coff_add_segment         (const char *);
 extern void obj_coff_section             (int);
-extern void c_dot_file_symbol            (const char *, int);
 extern segT s_get_segment                (symbolS *);
 #ifndef tc_coff_symbol_emit_hook
 extern void tc_coff_symbol_emit_hook     (symbolS *);
